// Slider1.cpp : implementation file
//

#include "stdafx.h"
#include "Slider.h"
#include "Slider1.h"
#include "lsapi.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSlider

IMPLEMENT_DYNCREATE(CSlider, CWnd);

#define MOUSE_OVER_TIMER_ID		1

CSlider::CSlider()
{
	m_NormalBitmapPath		= _T("");
	m_MouseOverBitmapPath	= _T("");
	m_PressedBitmapPath		= _T("");
	m_MouseOverSound		= _T("");
	m_PressSound			= _T("");
	m_cDescription			= _T("");

	m_hRgn = NULL;

	m_PtPosition.x = 0;
	m_PtPosition.y = 0;

	m_bMouseTracker		= FALSE;
	m_bChildIsExpanded	= FALSE;
	m_bTasks			= FALSE;
	m_bTopMost			= FALSE;
	m_bDrawMouseOver	= FALSE;
}

CSlider::~CSlider()
{
	// Destroy the Child
	m_Child.DestroyWindow();
}


BEGIN_MESSAGE_MAP(CSlider, CWnd)
	//{{AFX_MSG_MAP(CSlider)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_SETCURSOR()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_CHILD_CONTRACTED, OnChildContracted)
	ON_MESSAGE(WM_RUNBANG, OnRunBang)

END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CSlider message handlers

BOOL CSlider::CreateSlider(int nSliderNbr, HWND hParent)
{
	CString cErrorText = _T("");

	// Load all of our settings
	if(!LoadSettings(nSliderNbr))
		return FALSE;

	// Load the main bitmap, this has to be here
	if(!LoadBitmap(m_NormalBitmap, m_NormalBitmapPath))
	{
		cErrorText.Format("Unable to load bitmap - %s", m_NormalBitmapPath);

		ErrorLog(cErrorText);

		return FALSE;
	}

	// Load the pressed
	LoadBitmap(m_PressedBitmap, m_PressedBitmapPath);

	// Load the Mouse Over
	LoadBitmap(m_MouseOverBitmap, m_MouseOverBitmapPath);

	CString txtClassName = AfxRegisterWndClass(CS_DBLCLKS);		

	BITMAP bm;
	m_NormalBitmap.GetBitmap(&bm);

	UINT nStyleEx = WS_EX_TOOLWINDOW;	

	if(GetProfileIntValue(nSliderNbr, "AlwaysOnTop", 1))
	{
		nStyleEx |= WS_EX_TOPMOST;
		m_bTopMost = TRUE;
	}

	UINT nStyle = WS_POPUP;	

	if(!GetProfileIntValue(nSliderNbr, "StartUpHidden", 0))
	{
		nStyle |= WS_VISIBLE;
	}

	if(CreateEx(nStyleEx, txtClassName, "LsLnkMenu", nStyle, m_PtPosition.x, m_PtPosition.y, bm.bmWidth, bm.bmHeight, hParent, 0))
	{
		// Make sure window stays put
		SetWindowLong(m_hWnd, GWL_USERDATA, magicDWord);

		if(m_cDescription.GetLength() && m_bToolTip)
		{
			if(m_ToolTip.Create(this, TRUE))
				m_ToolTip.AddTool(this, m_cDescription);
		}

		return m_Child.CreateChildSlider(nSliderNbr, m_hWnd);
	}

	return FALSE;
}

BOOL CSlider::LoadSettings(int nSliderNbr)
{
	// Load all of our settings
	m_NormalBitmapPath = GetProfileStringValue(nSliderNbr, "MainBitmap");
	
	if(!m_NormalBitmapPath.GetLength())
	{
		ErrorLog("No MainBitmap entry in INI file!");
		return FALSE;
	}

	m_PressedBitmapPath		= GetProfileStringValue(nSliderNbr, "MainPressedBitmap");
	
	m_MouseOverBitmapPath	= GetProfileStringValue(nSliderNbr, "MainOverBitmap");

	// Do any conversions for the position
	m_PtPosition = ConvertToScreenPosition(GetProfileIntValue(nSliderNbr, "XPos", 0),
										   GetProfileIntValue(nSliderNbr, "YPos", 0));


	m_cDescription		= GetProfileStringValue(nSliderNbr, "Description");
	m_MouseOverSound	= GetProfileStringValue(nSliderNbr, "PanelMouseOverSound", "");
	m_PressSound		= GetProfileStringValue(nSliderNbr, "PanelPressSound"	 , "");

	// No tooltips if no description
	if(m_cDescription.GetLength())
		m_bToolTip = GetProfileIntValue(nSliderNbr, "MainToolTip", 1);
	else
		m_bToolTip = FALSE;

	m_bTasks = GetProfileIntValue(nSliderNbr, "TASKS", 0);

	return TRUE;
}

void CSlider::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CDC memDC;
	memDC.CreateCompatibleDC(&dc);
	
	// First see if the child slider is expanded
	if(m_bChildIsExpanded && m_PressedBitmap.m_hObject)
	{
		DrawPressed(&dc);
		return;
	}

	if(m_bDrawMouseOver)
	{
		DrawMouseOver(&dc);
	}
	else
	{
		m_hRgn = BitmapToRegion(m_NormalBitmap, RGB(255,0,255), 0, 0, 0);

		if(m_hRgn)
			SetWindowRgn(m_hRgn, FALSE);

		memDC.SelectObject(&m_NormalBitmap);

		BITMAP bm;
		m_NormalBitmap.GetBitmap(&bm);

		dc.BitBlt(0, 0, bm.bmWidth, bm.bmHeight, &memDC, 0, 0, SRCCOPY);
	}
}

void CSlider::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Play the sound
	if(!m_bChildIsExpanded && m_PressSound.GetLength())
		PlaySound(m_PressSound, NULL, SND_ASYNC | SND_FILENAME);

	m_Child.OnParentMove(m_hWnd);

	m_Child.Animate(10);	

	CWnd::OnLButtonDown(nFlags, point);

	m_bChildIsExpanded = m_Child.IsExpanded();

	Invalidate();
}

void CSlider::OnLButtonUp(UINT nFlags, CPoint point) 
{
	Invalidate();
	
	CWnd::OnLButtonUp(nFlags, point);
}

void CSlider::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnLButtonDown(nFlags, point);
}

void CSlider::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_bToolTip)
	{
		// The RelayEvent needs screen coordinates
		CPoint ScreenPt = point;
		ClientToScreen(&ScreenPt);

		MSG TooltipMessage;

		TooltipMessage.hwnd		= m_hWnd;
		TooltipMessage.message	= WM_MOUSEMOVE;
		TooltipMessage.wParam	= nFlags;
		TooltipMessage.lParam	= MAKELPARAM(ScreenPt.x, ScreenPt.y);
		TooltipMessage.pt		= ScreenPt;

		m_ToolTip.RelayEvent(&TooltipMessage);
	}

	if(!m_bMouseTracker)
	{
		m_bDrawMouseOver = TRUE;

		m_bMouseTracker = TRUE;

		// Play the sound
		if(!m_bChildIsExpanded && m_MouseOverSound.GetLength())
			PlaySound(m_MouseOverSound, NULL, SND_ASYNC | SND_FILENAME);

		// set the time
		SetTimer(MOUSE_OVER_TIMER_ID, 50, NULL);

		Invalidate();
	}
	
}

LRESULT CSlider::OnChildContracted(WPARAM wParam, LPARAM lParam)
{
	m_bChildIsExpanded = FALSE;
	Invalidate();
	
	return 1;
}

void CSlider::DrawPressed(CDC* pDC)
{
	// Draw the button in hte pressed state if it is there
	if(!m_PressedBitmap.m_hObject)
		return;

	CDC memDC;

	m_hRgn = BitmapToRegion(m_PressedBitmap, RGB(255,0,255), 0, 0, 0);

	if(m_hRgn)
		SetWindowRgn(m_hRgn, FALSE);

	memDC.CreateCompatibleDC(pDC);
	memDC.SelectObject(&m_PressedBitmap);

	BITMAP bm;
	m_PressedBitmap.GetBitmap(&bm);

	pDC->BitBlt(0, 0, bm.bmWidth, bm.bmHeight, &memDC, 0, 0, SRCCOPY);
}

void CSlider::DrawMouseOver(CDC* pDC)
{
	// Draw the button with the mouse over bitmap
	if(m_bChildIsExpanded)
		return;

	if(!m_MouseOverBitmap.m_hObject)
		return;

	CDC memDC;

	m_hRgn = BitmapToRegion(m_MouseOverBitmap, RGB(255,0,255), 0, 0, 0);

	if(m_hRgn)
		SetWindowRgn(m_hRgn, FALSE);

	memDC.CreateCompatibleDC(pDC);
	memDC.SelectObject(&m_MouseOverBitmap);

	BITMAP bm;
	m_MouseOverBitmap.GetBitmap(&bm);

	pDC->BitBlt(0, 0, bm.bmWidth, bm.bmHeight, &memDC, 0, 0, SRCCOPY);
}

void CSlider::OnTimer(UINT nIDEvent) 
{
	if(MOUSE_OVER_TIMER_ID == nIDEvent)
	{
		//Check our mouse pos and redraw if necessary
		POINT curPT;
		GetCursorPos(&curPT);
					 
		RECT rect;
		GetWindowRect(&rect);
		
		if(!PtInRect(&rect, curPT) )
		{
			KillTimer(MOUSE_OVER_TIMER_ID);

			m_bMouseTracker = FALSE;

			if(m_bToolTip)
			{
				// Get rid of any ToolTip
				MSG TooltipMessage;

				TooltipMessage.hwnd		= m_hWnd;
				TooltipMessage.message	= WM_MOUSEMOVE;
				TooltipMessage.wParam	= 0;
				TooltipMessage.lParam	= MAKELPARAM(curPT.x, curPT.y);
				TooltipMessage.pt		= curPT;

				m_ToolTip.RelayEvent(&TooltipMessage);
			}

			m_bDrawMouseOver = FALSE;

			Invalidate();
			UpdateWindow();
		}	
	}
	
	CWnd::OnTimer(nIDEvent);
}

BOOL CSlider::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	return CWnd::OnSetCursor(pWnd, nHitTest, message);
}

void CSlider::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);
}

void CSlider::ResetPanel()
{
	// Called from the App in response to the lslnkResetPanel bang command
	m_Child.ResetPanel();
}

void CSlider::Execute(int nBtnNbr)
{
	m_Child.Execute(nBtnNbr);
}

void CSlider::Execute(CString cBtnDesc)
{
	m_Child.Execute(cBtnDesc);
}

void CSlider::HideMenu()
{
	m_Child.Animate(10);

	m_bChildIsExpanded = m_Child.IsExpanded();

	Invalidate();
}
BOOL CSlider::IsExpanded()
{
	return m_bChildIsExpanded;
}
void CSlider::ShowMenu()
{
	// Called to show our child

	// Play the sound
	if(!m_bChildIsExpanded && m_PressSound.GetLength())
		PlaySound(m_PressSound, NULL, SND_ASYNC | SND_FILENAME);

	m_Child.Animate(10);	

	m_bChildIsExpanded = m_Child.IsExpanded();

	Invalidate();
}

LRESULT CSlider::OnRunBang(WPARAM wParam, LPARAM lParam)
{
	try
	{
		// Type definitions for pointer to ParseBangCommand
		typedef BOOL (_cdecl *PARSEBANGCOMMAND)(HWND, char*, char*);

		PARSEBANGCOMMAND fpnParseBangCommand = NULL;

		fpnParseBangCommand = (PARSEBANGCOMMAND)GetProcAddress(GetModuleHandle("LSAPI.DLL"), "ParseBangCommand");

		if(!fpnParseBangCommand)
			return 0;

		CString cCmdLine	= _T("");

		// CommandLine is the title
		GetWindowText(cCmdLine);

		CString cCmd		= _T("");
		CString cArgs		= _T("");

		int nPos = cCmdLine.Find(" ", 0);
		
		if(-1 != nPos)
		{
			cCmd	= cCmdLine.Left(nPos);
			cArgs	= cCmdLine.Mid(nPos + 1);

			fpnParseBangCommand(m_hWnd, cCmd.GetBuffer(cCmd.GetLength()), cArgs.GetBuffer(cArgs.GetLength()));
		}
		else
			fpnParseBangCommand(m_hWnd, cCmdLine.GetBuffer(cCmdLine.GetLength()), NULL);
	}
	catch(...)
	{
		TRACE0("Error\n");
	}

	return 1;
}

void CSlider::Move(int nXPos, int nYPos)
{
	// Called from theApp to move us
	CPoint ScreenPoint = ConvertToScreenPosition(nXPos, nYPos);

	// Move it
	SetWindowPos(NULL, ScreenPoint.x, ScreenPoint.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);

	// Tell our child to move
	m_Child.OnParentMove(m_hWnd);
}


void CSlider::BoxIt(HWND hwndBox)
{
	CWnd box;
	box.Attach(hwndBox);
	SetParent(&box);

	m_Child.OnParentMove(m_hWnd);
	box.Detach();
}