#include "stdafx.h"
#include "Slider.h"
#include "common.h"

extern CSliderApp theApp;

void ErrorLog(CString cErrorText)
{
	// Log error info
	CStdioFile f;

	CString cFilePath = theApp.m_IniFile.Left(theApp.m_IniFile.GetLength() - 13);
	cFilePath += "LsLnkError.txt";

	CString cText = _T("");

	if(f.Open(cFilePath, CFile::modeCreate | CFile::modeNoTruncate | CFile::modeWrite))
	{
		f.SeekToEnd();

		cText.Format("%s  %s", CTime::GetCurrentTime().Format("%c"), cErrorText);

		cText += '\n';
		cText += '\n';

		f.WriteString(cText);

		f.Close();
	}
}

BOOL GetLinkIcon( const CString & strLinkName, CString & strPath, int& nIconIndex ) 
{
	HRESULT hres;
	IShellLink * psl;
 
	// Call CoCreateInstance to obtain the IShellLink
	// Interface pointer. This call fails if
	// CoInitialize is not called, so it is assumed that
	// CoInitialize has been called.
	//AfxOleInit();

	hres =	CoCreateInstance(
				CLSID_ShellLink,
				0,
				CLSCTX_INPROC_SERVER,
				IID_IShellLink,
				(LPVOID *)&psl
			);

	if( SUCCEEDED(hres) ) {
		
		IPersistFile * ppf;
 
		// The IShellLink Interface supports the IPersistFile
		// interface. Get an interface pointer to it.
		hres =	psl->QueryInterface(
					IID_IPersistFile,
					(LPVOID *)&ppf
				);
		if( SUCCEEDED( hres ) ) {
			WORD wsz[MAX_PATH];
 
			//Convert the given link name string to wide character string.
#ifdef _UNICODE
			_tcsncpy(wsz, strLinkName, MAX_PATH);
#else
			MultiByteToWideChar(
				CP_ACP,
				0,
				strLinkName,
				-1,
				wsz,
				MAX_PATH
			);
#endif // _UNICODE

			//Load the file.
			hres = ppf->Load( wsz, STGM_READ );
			if( SUCCEEDED( hres ) ) {
				// Resolve the link by calling the Resolve() interface function.
				hres =	psl->Resolve(
							0,
							SLR_ANY_MATCH | SLR_NO_UI
						);
				if( SUCCEEDED( hres ) ) {
					TCHAR chPath[MAX_PATH];
					hres =	psl->GetIconLocation(
								chPath,
								MAX_PATH,
								&nIconIndex
							);
					if( SUCCEEDED( hres ) ) {
						// check wether the string needs expansion
 						TCHAR szPath[MAX_PATH];
						strPath = chPath;
						
						// If strPath is blank, then the icon is from the
						// executable file itself
						if(!strPath.GetLength())
							GetLinkPath(strLinkName, strPath);
					
						if( strPath.Find( TEXT('%') ) >= 0 ) {
							ExpandEnvironmentStrings( chPath, szPath, MAX_PATH );
							strPath = szPath;
						}
					}
				}
			}
			ppf->Release();
		}
		psl->Release();
	}
	return SUCCEEDED(hres) ? TRUE : FALSE;
}

// CSlider :: GetLinkPath() fills the filename and path buffer
// with relevant information
// strLinkName	- name of the link file passed into the function.
// strPath		- the buffer that will receive the filepath name.
 
BOOL GetLinkPath( const CString & strLinkName, CString & strPath ) 
{
	HRESULT hres;
	IShellLink * psl;
	WIN32_FIND_DATA wfd;
 
	// Call CoCreateInstance to obtain the IShellLink
	// Interface pointer. This call fails if
	// CoInitialize is not called, so it is assumed that
	// CoInitialize has been called.
 
	hres =	CoCreateInstance(
				CLSID_ShellLink,
				0,
				CLSCTX_INPROC_SERVER,
				IID_IShellLink,
				(LPVOID *)&psl
			);

	if( SUCCEEDED(hres) ) {
		IPersistFile * ppf;
 
		// The IShellLink Interface supports the IPersistFile
		// interface. Get an interface pointer to it.
		hres =	psl->QueryInterface(
					IID_IPersistFile,
					(LPVOID *)&ppf
				);
		if( SUCCEEDED( hres ) ) {
			WORD wsz[MAX_PATH];
 
			//Convert the given link name string to wide character string.
#ifdef _UNICODE
			_tcsncpy(wsz, strLinkName, MAX_PATH);
#else
			MultiByteToWideChar(
				CP_ACP,
				0,
				strLinkName,
				-1,
				wsz,
				MAX_PATH
			);
#endif // _UNICODE

			//Load the file.
			hres = ppf->Load( wsz, STGM_READ );
			if( SUCCEEDED( hres ) ) {
				// Resolve the link by calling the Resolve() interface function.
				hres =	psl->Resolve(
							0,
							SLR_ANY_MATCH | SLR_NO_UI
						);
				if( SUCCEEDED( hres ) ) {
					TCHAR chPath[MAX_PATH];
					hres =	psl->GetPath(
								chPath,
								MAX_PATH,
								(WIN32_FIND_DATA*)&wfd,
								SLGP_SHORTPATH
							);
					if( SUCCEEDED( hres ) ) {
						// check wether the string needs expansion
 						TCHAR szPath[MAX_PATH];
						strPath = chPath;
						if( strPath.Find( TEXT('%') ) >= 0 ) {
							ExpandEnvironmentStrings( chPath, szPath, MAX_PATH );
							strPath = szPath;
						}
					}
				}
			}
			ppf->Release();
		}
		psl->Release();
	}
	return SUCCEEDED(hres) ? TRUE : FALSE;
}

BOOL GetLinkDisplay( const CString & strLinkName, CString & strText )
{
	HRESULT hres;
	IShellLink * psl;
 
	// Call CoCreateInstance to obtain the IShellLink
	// Interface pointer. This call fails if
	// CoInitialize is not called, so it is assumed that
	// CoInitialize has been called.
 
	hres =	CoCreateInstance(
				CLSID_ShellLink,
				0,
				CLSCTX_INPROC_SERVER,
				IID_IShellLink,
				(LPVOID *)&psl
			);

	if( SUCCEEDED(hres) ) {
		IPersistFile * ppf;
 
		// The IShellLink Interface supports the IPersistFile
		// interface. Get an interface pointer to it.
		hres =	psl->QueryInterface(
					IID_IPersistFile,
					(LPVOID *)&ppf
				);
		if( SUCCEEDED( hres ) ) {
			WORD wsz[MAX_PATH];
 
			//Convert the given link name string to wide character string.
#ifdef _UNICODE
			_tcsncpy(wsz, strLinkName, MAX_PATH);
#else
			MultiByteToWideChar(
				CP_ACP,
				0,
				strLinkName,
				-1,
				wsz,
				MAX_PATH
			);
#endif // _UNICODE

			//Load the file.
			hres = ppf->Load( wsz, STGM_READ );
			if( SUCCEEDED( hres ) ) {
				// Resolve the link by calling the Resolve() interface function.
				hres =	psl->Resolve(
							0,
							SLR_ANY_MATCH | SLR_NO_UI
						);
				if( SUCCEEDED( hres ) ) {
					TCHAR chDisplay[MAX_PATH];
					hres =	psl->GetDescription(chDisplay, MAX_PATH);

					if( SUCCEEDED( hres ) ) 
						strText = chDisplay;
				}
			}
			ppf->Release();
		}
		psl->Release();
	}
	return SUCCEEDED(hres) ? TRUE : FALSE;
}

int GetNumberOfLinkFilesInDirectory(CString cPath)
{
	int nRtnAmount = 0;

	WIN32_FIND_DATA  FindData;

	HANDLE hFindHandle = FindFirstFile(cPath, &FindData);

	for(;;)
	{
		if(INVALID_HANDLE_VALUE == hFindHandle)
			break;
		else
			nRtnAmount++;

		if(!FindNextFile(hFindHandle, &FindData))
			break;
	}

	FindClose(hFindHandle);

	return nRtnAmount;

}

int GetLinkFilesInDirectory(CString cPath, CStringArray& strArray, CString cExclusionString)
{
	// Fill array up with the .lnk files names and return number of them
	CString cFileName = _T("");
	
	strArray.RemoveAll();

	WIN32_FIND_DATA  FindData;

	HANDLE hFindHandle = FindFirstFile(cPath, &FindData);

	for(;;)
	{
		if(INVALID_HANDLE_VALUE == hFindHandle)
			break;
		
		if(!(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		{
			cFileName = FindData.cFileName;

			cFileName.MakeUpper();

			// Make sure its not in the exclusion list
			if(cExclusionString.Find(cFileName.Right(4)) != -1)
				// Not allowed
				TRACE1("Skipping %s\n", cFileName);
			else
				// Add it
				strArray.Add(FindData.cFileName);
		}

		if(!FindNextFile(hFindHandle, &FindData))
			break;
	}

	FindClose(hFindHandle);

	return strArray.GetSize();
}


BOOL LoadBitmap(CBitmap& pBitmap, CString cBitmapFile)
{
	//AFX_MANAGE_STATE(AfxGetStaticModuleState());
	// Load the main bitmaps if they aren't already

	if(!pBitmap.m_hObject)
	{
		HBITMAP hMainBM = LoadLSImage(cBitmapFile, NULL);/*(HBITMAP) LoadImage(AfxGetInstanceHandle(), cBitmapFile,
							IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR | LR_LOADFROMFILE | LR_CREATEDIBSECTION );*/
		
		if(hMainBM)
		{
			pBitmap.Attach(hMainBM);
			return TRUE;
		}
		else
			return FALSE;
	}
	else
		return TRUE;

}

void DrawSemiTransparentBitmap(CDC *pDstDC, int x, int y, int nWidth, int nHeight, CDC* pSrcDC, int xSrc, int ySrc, COLORREF clrFade)
{
    CDC dcCompatible;
    CBitmap *pBitmapOld;    
	CBitmap bm;

    dcCompatible.CreateCompatibleDC(pDstDC);
    bm.CreateCompatibleBitmap(pDstDC, nWidth, nHeight);

    pBitmapOld = dcCompatible.SelectObject(&bm);
    dcCompatible.FillSolidRect(CRect(0, 0, nWidth, nHeight), clrFade);

    pDstDC->BitBlt(x, y, nWidth, nHeight, &dcCompatible, 0, 0, SRCAND);

    dcCompatible.SelectObject(pBitmapOld);
    pDstDC->BitBlt(x, y, nWidth, nHeight, pSrcDC, 0, 0, SRCPAINT);
}

int GetProfileIntValue(int nSliderNbr, CString cSetting, int nDefault)
{
	// Helper to get a Value from the Format file and return the value. If the entry doesn't
	// exist, the default section will be searched
	
	CString cSection	= _T("");
	CString cDefault    = _T("");
	CString cValue      = _T("");
	TCHAR   temp[MAX_PATH];
	cSection.Format("Slider%d", nSliderNbr);
	cDefault.Format("%d", nDefault);
	
	int nRtnValue = nDefault;

	cValue = GetProfileStringValue(nSliderNbr, cSetting, cDefault);
	if ( cValue == _T("") || cValue.GetLength() < 1)
		nRtnValue = 0;
	else
		nRtnValue = _ttoi(cValue.GetBuffer(cValue.GetLength()));

	//nRtnValue = GetPrivateProfileInt(cSection, cSetting, nDefault, theApp.m_IniFile);

	// Try the current section first
	if(nDefault == nRtnValue) {
		// Hit the default section
		if ( GetPrivateProfileString("Default", cSetting, cDefault, temp, MAX_PATH, theApp.m_IniFile) ) {
			
			CString cTemp = temp;
			VarExpansion(temp, cTemp.GetBuffer(MAX_PATH));
			cValue = temp;
			nRtnValue = _ttoi(cValue.GetBuffer(cValue.GetLength()));
			if ( cValue == _T("")  || cValue.GetLength() < 1)
				nRtnValue = 0;
			else
				nRtnValue = _ttoi(cValue.GetBuffer(cValue.GetLength()));
		}
		else {
			nRtnValue = nDefault;
		}

	}
	
		return nRtnValue;
}

CString GetProfileStringValue(int nSliderNbr, CString cSetting, CString cDefault)
{
	// Helper to get a Value from the Format file and return the value. If the entry doesn't
	// exist, the default section will be searched
	CString cSection	= _T("");
	CString cRtnString	= _T("");
	TCHAR temp[MAX_PATH];

	cSection.Format("Slider%d", nSliderNbr);

	if(!GetPrivateProfileString(cSection, cSetting, "", cRtnString.GetBuffer(MAX_PATH), MAX_PATH, theApp.m_IniFile))
	{
		cRtnString.ReleaseBuffer();

		// Hit the Default Section
		if(!GetPrivateProfileString("Default", cSetting, cDefault, cRtnString.GetBuffer(MAX_PATH), MAX_PATH, theApp.m_IniFile))
		{
			//AfxMessageBox(cDefault);
			cRtnString.ReleaseBuffer();
			return cDefault;
		}	
	}

	VarExpansion(temp, cRtnString.GetBuffer(MAX_PATH));
	cRtnString = temp;

	//AfxMessageBox(cRtnString);
	cRtnString.ReleaseBuffer();

	return cRtnString;
}

// CStringArray Sorting Methods
void SortStringArray (CStringArray& csa, BOOL bDescending, BOOL bCase)
{
	int iArraySize = csa.GetSize();  

	if (iArraySize <= 0)     
		return;
	
	int iCSSize = sizeof (CString*);  
	
	void* pArrayStart = (void *)&csa[0];

	if (bDescending)
		if(bCase)
			qsort (pArrayStart, iArraySize, iCSSize, CompareDescending);  
		else
			qsort (pArrayStart, iArraySize, iCSSize, CompareDescendingNoCase); 
	else
		if(bCase)
			qsort (pArrayStart, iArraySize, iCSSize, CompareAscending);
		else
			qsort (pArrayStart, iArraySize, iCSSize, CompareAscendingNoCase);
}

int CompareAscending(const void *a, const void *b)
{
	CString *pA = (CString*)a;
	CString *pB = (CString*)b;  
	
	return (pA->Compare(*pB));
}

int CompareDescending(const void *a, const void *b)
{
	CString *pA = (CString*)a;
	CString *pB = (CString*)b;  
	
	return (-1 * (pA->Compare(*pB)));
}
int CompareAscendingNoCase(const void *a, const void *b)
{
	CString *pA = (CString*)a;
	CString *pB = (CString*)b;  
	
	return (pA->CompareNoCase(*pB));
}

int CompareDescendingNoCase(const void *a, const void *b)
{
	CString *pA = (CString*)a;
	CString *pB = (CString*)b;  
	
	return (-1 * (pA->CompareNoCase(*pB)));
}

CPoint ConvertToScreenPosition(int nXPos, int nYPos)
{
	// this will take 2 points and return a CPoint that is a point on the screen

	// If the values are negative, then they are in relation to the screen.
	// Example nXPos = -50. This means 50 from the right side. If they are positive,
	// they are relative to the left and top. Example nXPos = 20. This means
	// 20 from the left side (or just 20)

	CPoint RtnPoint;

	RtnPoint.x = 0;
	RtnPoint.y = 0;

	// Get the screen coordinates
	int nScreenWidth = GetSystemMetrics(SM_CXSCREEN);
	int nScreenHeight= GetSystemMetrics(SM_CYSCREEN);

	if(nXPos < 0)
		// Ths value is relative to the right side
		RtnPoint.x = nScreenWidth + nXPos;
	else
		RtnPoint.x = nXPos;

	if(nYPos < 0)
		// This value is relative to the bottom
		RtnPoint.y = nScreenHeight + nYPos;
	else
		RtnPoint.y = nYPos;

	return RtnPoint;

}

